﻿using FactPDA.Models;
using System;
using System.Collections;
using System.Text;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace FactPDA.Views
{
    public class RadioButtonEx : ContentView
    {
        // 변경하고 싶은데 안됨
        // 참고사이트 : https://www.alt-codes.net/circle-symbols
        static readonly string checkOff = "\x25CB";
        static readonly string checkOn = "\u25C9";
        Label checkLabel;

        public RadioButtonEx()
        {
            // Assemble the view.
            checkLabel = new Label
            {
                Text = checkOff
            };

            Label textLabel = new Label();
            this.Content = new StackLayout
            {
                Orientation = StackOrientation.Horizontal,
                Children =
                {
                    checkLabel,
                    textLabel
                }
            };

            // Set the bindings.
            checkLabel.BindingContext = this;
            checkLabel.SetBinding(Label.FontProperty, "Font");
            checkLabel.SetBinding(Label.FontSizeProperty, "FontSize");
            checkLabel.SetBinding(Label.TextColorProperty, "TextColor");
            checkLabel.SetBinding(Label.BackgroundColorProperty, "BackgroundColor");

            textLabel.BindingContext = this;
            textLabel.SetBinding(Label.TextProperty, "Text");
            textLabel.SetBinding(Label.TextColorProperty, "TextColor");
            textLabel.SetBinding(Label.FontProperty, "Font");
            textLabel.SetBinding(Label.FontSizeProperty, "FontSize");
            textLabel.SetBinding(Label.BackgroundColorProperty, "BackgroundColor");

            // Install a Tap recognizer.
            TapGestureRecognizer recognizer = new TapGestureRecognizer
            {
                Parent = this,
                NumberOfTapsRequired = 1
            };
            recognizer.Tapped += (ssender, args) =>
            {
                this.IsToggled = true;
            };

            this.GestureRecognizers.Add(recognizer);
        }

        // Define the Text bindable property and property.
        public static readonly BindableProperty TextProperty =
            BindableProperty.Create("Text",                 // property name
                                    typeof(string),         // property type
                                    typeof(RadioButtonEx),    // this type
                                    null);                  // initial value

        public string Text
        {
            set { SetValue(TextProperty, value); }
            get { return (string)GetValue(TextProperty); }
        }

        // Define the TextColor bindable property and property.
        public static readonly BindableProperty TextColorProperty =
            BindableProperty.Create(nameof(TextColor),                   // property name
                                    typeof(Color),
                                    typeof(RadioButtonEx),
                                    Color.Default);
        public Color TextColor
        {
            set { SetValue(TextColorProperty, value); }
            get { return (Color)GetValue(TextColorProperty); }
        }

        // Define the Font bindable property and property.
        public static readonly BindableProperty FontProperty =
            BindableProperty.Create("Font",
                                    typeof(Font),
                                    typeof(RadioButtonEx),
                                    Font.SystemFontOfSize(NamedSize.Large));
        public Font Font
        {
            get { return (Font)GetValue(FontProperty); }
            set { SetValue(FontProperty, value); }
        }


        public static readonly BindableProperty FontSizeProperty =
            BindableProperty.Create(nameof(FontSize),
                                    typeof(double),
                                    typeof(RadioButtonEx),
                                    Device.GetNamedSize(NamedSize.Default, typeof(Label)));

        public double FontSize
        {
            get { return (double)GetValue(FontSizeProperty); }
            set { SetValue(FontSizeProperty, value); }
        }


        // Define the Color bindable property and property.
        public static readonly BindableProperty BackgroundColorProperty =
            BindableProperty.Create("BackgroundColor",
                                    typeof(Color),
                                    typeof(RadioButtonEx),
                                    Color.Default);

        public Color BackgroundColor
        {
            get { return (Color)GetValue(BackgroundColorProperty); }
            set { SetValue(BackgroundColorProperty, value); }
        }

        // Define the Toggled event, IsToggled bindable property and property.
        public event EventHandler<ToggledEventArgs> Toggled;

        public static readonly BindableProperty IsToggledProperty =
            BindableProperty.Create("IsToggled",
                                    typeof(bool),           // property type 
                                    typeof(RadioButtonEx),    // this type
                                    false,                  // initial value
                                    BindingMode.TwoWay,     // default binding
                                    null,                   // validation
                                    OnIsToggledPropertyChanged);

        public bool IsToggled
        {
            get { return (bool)GetValue(IsToggledProperty); }
            set { SetValue(IsToggledProperty, value); }
        }

        // Property changed handler for the IsToggled property.
        static void OnIsToggledPropertyChanged(BindableObject sender,
                                                object oldValue, object newValue)
        {
            // Get the object whose property is changing.
            RadioButtonEx radioButtonEx = (RadioButtonEx)sender;
            radioButtonEx.OnIsToggledPropertyChanged((bool)oldValue, (bool)newValue);
        }

        void OnIsToggledPropertyChanged(bool oldValue, bool newValue)
        {
            this.checkLabel.Text = newValue ? checkOn : checkOff;

            // Fire the event.
            if (this.Toggled != null)
            {
                this.Toggled(this, new ToggledEventArgs(this.IsToggled));
            }

            // If toggled on, toggle off all siblings.
            if (this.IsToggled)
            {
                Layout<View> parent = this.ParentView as Layout<View>;

                if (parent != null)
                {
                    foreach (View view in parent.Children)
                    {
                        if (view is RadioButtonEx && view != this)
                        {
                            ((RadioButtonEx)view).IsToggled = false;
                        }
                    }
                }
            }
        }
    }
}